@@ -173,8 +173,12 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags,
 			return fmt.Errorf("cannot apply '%d' migration files", count)
 		}
 	}
+	dirURL, err := url.Parse(flags.dirURL)
+	if err != nil {
+		return fmt.Errorf("parse dir-url: %w", err)
+	}
 	// Open and validate the migration directory.
-	dir, err := cmdmigrate.Dir(ctx, flags.dirURL, false)
+	dir, err := cmdmigrate.DirURL(ctx, dirURL, false)
 	if err != nil {
 		return err
 	}
@@ -217,7 +221,7 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags,
 		return err
 	}
 	// Setup reporting info.
-	report := cmdlog.NewMigrateApply(ctx, client, dir)
+	report := cmdlog.NewMigrateApply(ctx, client, dirURL)
 	mr.Init(client, report, mrrw)
 	// If cloud reporting is enabled, and we cannot obtain the current
 	// target identifier, abort and report it to the user.
@@ -639,102 +643,6 @@ an HCL, SQL, or ORM schema. See: https://atlasgo.io/versioned/diff`,
 	return cmd
 }
 
-func migrateDiffRun(cmd *cobra.Command, args []string, flags migrateDiffFlags, env *Env) error {
-	ctx := cmd.Context()
-	dev, err := sqlclient.Open(ctx, flags.devURL)
-	if err != nil {
-		return err
-	}
-	defer dev.Close()
-	// Acquire a lock.
-	if l, ok := dev.Driver.(schema.Locker); ok {
-		unlock, err := l.Lock(ctx, "atlas_migrate_diff", flags.lockTimeout)
-		if err != nil {
-			return fmt.Errorf("acquiring database lock: %w", err)
-		}
-		// If unlocking fails notify the user about it.
-		defer func() { cobra.CheckErr(unlock()) }()
-	}
-	// Open the migration directory.
-	u, err := url.Parse(flags.dirURL)
-	if err != nil {
-		return err
-	}
-	dir, err := cmdmigrate.DirURL(ctx, u, false)
-	if err != nil {
-		return err
-	}
-	if err := checkDirRebased(ctx, cmd, dir); err != nil {
-		return err
-	}
-	if flags.edit {
-		dir = &editDir{dir}
-	}
-	var name, indent string
-	if len(args) > 0 {
-		name = args[0]
-	}
-	f, err := cmdmigrate.Formatter(u)
-	if err != nil {
-		return err
-	}
-	if f, indent, err = mayIndent(u, f, flags.format); err != nil {
-		return err
-	}
-	// If there is a state-loader that requires a custom
-	// 'migrate diff' handling, offload it the work.
-	if d, ok := cmdext.States.Differ(flags.desiredURLs); ok {
-		err := d.MigrateDiff(ctx, &cmdext.MigrateDiffOptions{
-			To:      flags.desiredURLs,
-			Name:    name,
-			Indent:  indent,
-			Dir:     dir,
-			Dev:     dev,
-			Options: diffOptions(cmd, env),
-		})
-		return maskNoPlan(cmd, err)
-	}
-	// Get a state reader for the desired state.
-	desired, err := stateReader(ctx, env, &stateReaderConfig{
-		urls:    flags.desiredURLs,
-		dev:     dev,
-		client:  dev,
-		schemas: flags.schemas,
-		vars:    GlobalFlags.Vars,
-	})
-	if err != nil {
-		return err
-	}
-	defer desired.Close()
-	opts := []migrate.PlannerOption{
-		migrate.PlanFormat(f),
-		migrate.PlanWithIndent(indent),
-		migrate.PlanWithDiffOptions(diffOptions(cmd, env)...),
-	}
-	if dev.URL.Schema != "" {
-		// Disable tables qualifier in schema-mode.
-		opts = append(opts, migrate.PlanWithSchemaQualifier(flags.qualifier))
-	}
-	// Plan the changes and create a new migration file.
-	pl := migrate.NewPlanner(dev.Driver, dir, opts...)
-	plan, err := func() (*migrate.Plan, error) {
-		if dev.URL.Schema != "" {
-			return pl.PlanSchema(ctx, name, desired.StateReader)
-		}
-		return pl.Plan(ctx, name, desired.StateReader)
-	}()
-	var cerr *migrate.NotCleanError
-	switch {
-	case errors.As(err, &cerr) && dev.URL.Schema == "" && desired.Schema != "":
-		return fmt.Errorf("dev database is not clean (%s). Add a schema to the URL to limit the scope of the connection", cerr.Reason)
-	case err != nil:
-		return maskNoPlan(cmd, err)
-	default:
-		// Write the plan to a new file.
-		return pl.WritePlan(plan)
-	}
-}
-
 func mayIndent(dir *url.URL, f migrate.Formatter, format string) (migrate.Formatter, string, error) {
 	if format == "" {
 		return f, "", nil
@@ -964,6 +872,7 @@ type migrateLintFlags struct {
 // migrateLintCmd represents the 'atlas migrate lint' subcommand.
 func migrateLintCmd() *cobra.Command {
 	var (
+		env   *Env
 		flags migrateLintFlags
 		cmd   = &cobra.Command{
 			Use:   "lint [flags]",
@@ -972,14 +881,17 @@ func migrateLintCmd() *cobra.Command {
   atlas migrate lint --dir "file:///path/to/migrations" --dev-url "docker://mysql/8/dev" --latest 1
   atlas migrate lint --dir "file:///path/to/migrations" --dev-url "mysql://root:pass@localhost:3306" --git-base master
   atlas migrate lint --dir "file:///path/to/migrations" --dev-url "mysql://root:pass@localhost:3306" --format '{{ json .Files }}'`,
-			PreRunE: func(cmd *cobra.Command, args []string) error {
-				if err := migrateFlagsFromConfig(cmd); err != nil {
+			PreRunE: func(cmd *cobra.Command, args []string) (err error) {
+				if env, err = selectEnv(cmd); err != nil {
+					return err
+				}
+				if err := setMigrateEnvFlags(cmd, env); err != nil {
 					return err
 				}
 				return dirFormatBC(flags.dirFormat, &flags.dirURL)
 			},
 			RunE: RunE(func(cmd *cobra.Command, args []string) error {
-				return migrateLintRun(cmd, args, flags)
+				return migrateLintRun(cmd, args, flags, env)
 			}),
 		}
 	)
@@ -1298,7 +1210,11 @@ func migrateStatusCmd() *cobra.Command {
 
 func migrateStatusRun(cmd *cobra.Command, _ []string, flags migrateStatusFlags) error {
 	ctx := cmd.Context()
-	dir, err := cmdmigrate.Dir(ctx, flags.dirURL, false)
+	dirURL, err := url.Parse(flags.dirURL)
+	if err != nil {
+		return fmt.Errorf("parse dir-url: %w", err)
+	}
+	dir, err := cmdmigrate.DirURL(ctx, dirURL, false)
 	if err != nil {
 		return err
 	}
@@ -1310,9 +1226,10 @@ func migrateStatusRun(cmd *cobra.Command, _ []string, flags migrateStatusFlags)
 	if err := checkRevisionSchemaClarity(cmd, client, flags.revisionSchema); err != nil {
 		return err
 	}
-	report, err := (&cmdmigrate.StatusReporter{
+	report, err := (&cmdlog.StatusReporter{
 		Client: client,
 		Dir:    dir,
+		DirURL: dirURL,
 		Schema: revisionSchemaName(client, flags.revisionSchema),
 	}).Report(ctx)
 	if err != nil {
@@ -1410,7 +1327,7 @@ func checkRevisionSchemaClarity(cmd *cobra.Command, c *sqlclient.Client, revisio
 	if c.URL.Schema != "" && revisionSchemaFlag == "" {
 		// If the schema does not contain a revision table, but we can find a table in the previous default schema,
 		// abort and tell the user to specify the intention.
-		opts := &schema.InspectOptions{Tables: []string{revision.Table}}
+		opts := &schema.InspectOptions{Tables: []string{revision.Table}, Mode: schema.InspectTables}
 		s, err := c.InspectSchema(cmd.Context(), "", opts)
 		var ok bool
 		switch {
