@@ -7,13 +7,10 @@ package cmdapi
 import (
 	"context"
 	"database/sql"
-	"encoding/base64"
 	"encoding/json"
 	"errors"
 	"fmt"
 	"io"
-	"net/http"
-	"net/http/httptest"
 	"net/url"
 	"os"
 	"os/exec"
@@ -23,7 +20,6 @@ import (
 	"testing"
 	"time"
 
-	"ariga.io/atlas/cmd/atlas/internal/cloudapi"
 	"ariga.io/atlas/cmd/atlas/internal/cmdlog"
 	migrate2 "ariga.io/atlas/cmd/atlas/internal/migrate"
 	"ariga.io/atlas/sql/migrate"
@@ -550,7 +546,7 @@ env "local" {
 			"--var", fmt.Sprintf("cache=%s", uuid.NewString()),
 		)
 		// Empty list is expanded to zero blocks.
-		require.EqualError(t, err, `env "local" not defined in project file`)
+		require.EqualError(t, err, `env "local" not defined in config file`)
 	})
 
 	t.Run("TemplateDir", func(t *testing.T) {
@@ -880,423 +876,6 @@ env "local" {
 	})
 }
 
-func TestMigrate_ApplyCloudReport(t *testing.T) {
-	var (
-		dir     migrate.MemDir
-		status  int
-		report  cloudapi.ReportMigrationInput
-		reports cloudapi.ReportMigrationSetInput
-		srv     = httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
-			var m struct {
-				Query     string `json:"query"`
-				Variables struct {
-					Input json.RawMessage `json:"input"`
-				} `json:"variables"`
-			}
-			require.NoError(t, json.NewDecoder(r.Body).Decode(&m))
-			switch {
-			case strings.Contains(m.Query, "query") && strings.Contains(m.Query, "me"):
-				fmt.Fprintln(w, `{"data": {"me":{ "name": "a8m", "org": "graph"}}}`)
-			case strings.Contains(m.Query, "query"):
-				// Checksum before archiving.
-				hf, err := dir.Checksum()
-				require.NoError(t, err)
-				ht, err := hf.MarshalText()
-				require.NoError(t, err)
-				require.NoError(t, dir.WriteFile(migrate.HashFileName, ht))
-				// Archive and send.
-				arc, err := migrate.ArchiveDir(&dir)
-				require.NoError(t, err)
-				fmt.Fprintf(w, `{"data":{"dirState":{"content":%q}}}`, base64.StdEncoding.EncodeToString(arc))
-			case strings.Contains(m.Query, "mutation") && strings.Contains(m.Query, "ReportMigrationSet"):
-				if status != 0 {
-					w.WriteHeader(status)
-				}
-				require.NoError(t, json.Unmarshal(m.Variables.Input, &reports))
-				fmt.Fprint(w, `{"data":{"reportMigrationSet":{"url": "https://gh.atlasgo.cloud/deployments/sets/94489280524"}}}`)
-			case strings.Contains(m.Query, "mutation") && strings.Contains(m.Query, "ReportMigration"):
-				if status != 0 {
-					w.WriteHeader(status)
-				}
-				require.NoError(t, json.Unmarshal(m.Variables.Input, &report))
-				fmt.Fprint(w, `{"data":{"reportMigration":{"url": "https://gh.atlasgo.cloud/deployments/51539607559"}}}`)
-			default:
-				t.Fatalf("unexpected query: %s", m.Query)
-			}
-		}))
-		h = `
-variable "cloud_url" {
-  type = string
-}
-
-atlas {
-  cloud {
-    token   = "token"
-    url     = var.cloud_url
-    project = "example"
-  }
-}
-
-data "remote_dir" "migrations" {
-  name = "migrations/v1/mysql"
-}
-
-env {
-  name = atlas.env
-  migration {
-    dir = data.remote_dir.migrations.url
-  }
-}
-`
-		u    = openSQLite(t, "")
-		p    = t.TempDir()
-		path = filepath.Join(p, "atlas.hcl")
-	)
-	t.Cleanup(srv.Close)
-	require.NoError(t, os.WriteFile(path, []byte(h), 0600))
-
-	t.Run("NoPendingFiles", func(t *testing.T) {
-		cmd := migrateCmd()
-		cmd.AddCommand(migrateApplyCmd())
-		s, err := runCmd(
-			cmd, "apply",
-			"-c", "file://"+path,
-			"--env", "local",
-			"--url", u,
-			"--var", "cloud_url="+srv.URL,
-		)
-		require.NoError(t, err)
-		require.Equal(t, "No migration files to execute\nhttps://gh.atlasgo.cloud/deployments/51539607559\n", s)
-		require.NotEmpty(t, report.Target.ID)
-		_, err = uuid.Parse(report.Target.ID)
-		require.NoError(t, err, "target id is not a valid uuid")
-		require.False(t, report.StartTime.IsZero())
-		require.False(t, report.EndTime.IsZero())
-		require.Nil(t, report.Context)
-		require.Equal(t, cloudapi.ReportMigrationInput{
-			ProjectName:  "example",
-			DirName:      "migrations/v1/mysql",
-			EnvName:      "local",
-			AtlasVersion: operatorVersion(),
-			StartTime:    report.StartTime,
-			EndTime:      report.EndTime,
-			Files:        []cloudapi.DeployedFileInput{},
-			Target: cloudapi.DeployedTargetInput{
-				ID:     report.Target.ID, // generated uuid
-				Schema: "main",
-				URL:    u,
-			},
-			Log: "No migration files to execute\n",
-		}, report)
-	})
-
-	t.Run("WithContext", func(t *testing.T) {
-		cmd := migrateCmd()
-		cmd.AddCommand(migrateApplyCmd())
-		s, err := runCmd(
-			cmd, "apply",
-			"-c", "file://"+path,
-			"--env", "local",
-			"--url", u,
-			"--var", "cloud_url="+srv.URL,
-			"--context", `{ "triggerType": "KUBERNETES", "triggerVersion": "v1.2.3" }`,
-		)
-		require.NoError(t, err)
-		require.Equal(t, "No migration files to execute\nhttps://gh.atlasgo.cloud/deployments/51539607559\n", s)
-		require.NotNil(t, report.Context)
-		require.Equal(t, report.Context.TriggerType, "KUBERNETES")
-		require.Equal(t, report.Context.TriggerVersion, "v1.2.3")
-	})
-
-	t.Run("WithFiles", func(t *testing.T) {
-		require.NoError(t, dir.WriteFile("1.sql", []byte("create table foo (id int)")))
-		require.NoError(t, dir.WriteFile("2.sql", []byte("create table bar (id int)")))
-
-		cmd := migrateCmd()
-		cmd.AddCommand(migrateApplyCmd())
-		s, err := runCmd(
-			cmd, "apply",
-			"-c", "file://"+path,
-			"--env", "local",
-			"--url", u,
-			"--var", "cloud_url="+srv.URL,
-			// Inject fake variable to enforce re-evaluation of the data source (skip cache).
-			"--var", fmt.Sprintf("cache=%s", uuid.NewString()),
-		)
-		require.NoError(t, err)
-		// Reporting does not affect the output.
-		require.True(t, strings.HasPrefix(s, "Migrating to version 2 (2 migrations in total):"))
-		require.True(t, strings.HasSuffix(s, "  -- 2 migrations\n  -- 2 sql statements\nhttps://gh.atlasgo.cloud/deployments/51539607559\n"))
-		require.Equal(t, "", report.FromVersion, "from empty database")
-		require.Equal(t, "2", report.ToVersion)
-		require.Equal(t, "2", report.CurrentVersion)
-		require.Len(t, report.Files, 2)
-		for i, n := range []string{"1.sql", "2.sql"} {
-			require.Equal(t, n, report.Files[i].Name)
-			require.Equal(t, 1, report.Files[i].Applied)
-			require.Zero(t, report.Files[i].Skipped)
-			require.Nil(t, report.Files[i].Error)
-		}
-	})
-
-	t.Run("PrintError", func(t *testing.T) {
-		status = http.StatusInternalServerError
-		require.NoError(t, dir.WriteFile("3.sql", []byte("create table baz (id int)")))
-		cmd := migrateCmd()
-		cmd.AddCommand(migrateApplyCmd())
-		s, err := runCmd(
-			cmd, "apply",
-			"-c", "file://"+path,
-			"--env", "local",
-			"--url", u,
-			"--var", "cloud_url="+srv.URL,
-			// Inject fake variable to enforce re-evaluation of the data source (skip cache).
-			"--var", fmt.Sprintf("cache=%s", uuid.NewString()),
-		)
-		require.NoError(t, err)
-		// Reporting error should not affect the migration execution.
-		require.True(t, strings.HasSuffix(s, "  -- 1 migration\n  -- 1 sql statement\nError: unexpected status code: 500\n"))
-
-		// Custom logging.
-		cmd = migrateCmd()
-		cmd.AddCommand(migrateApplyCmd())
-		s, err = runCmd(
-			cmd, "apply",
-			"-c", "file://"+path,
-			"--env", "local",
-			"--url", u,
-			"--var", "cloud_url="+srv.URL,
-			"--format", "{{ .Env.Driver }}",
-		)
-		require.NoError(t, err)
-		require.Equal(t, "sqlite3\nError: unexpected status code: 500\n", s)
-	})
-
-	t.Run("OpenError", func(t *testing.T) {
-		sqlclient.Register("openerror", sqlclient.OpenerFunc(func(ctx context.Context, u *url.URL) (*sqlclient.Client, error) {
-			return nil, errors.New("openerror")
-		}))
-		cmd := migrateCmd()
-		cmd.AddCommand(migrateApplyCmd())
-		_, err := runCmd(
-			cmd, "apply",
-			"-c", "file://"+path,
-			"--env", "local",
-			"--url", "openerror://db",
-			"--var", "cloud_url="+srv.URL,
-			"--context", `{ "triggerType": "KUBERNETES", "triggerVersion": "v1.2.3" }`,
-		)
-		require.EqualError(t, err, "openerror")
-		require.Equal(t, "Error: openerror", *reports.Error)
-		require.NotNil(t, reports.Context)
-		require.Equal(t, reports.Context.TriggerType, "KUBERNETES")
-		require.Equal(t, reports.Context.TriggerVersion, "v1.2.3")
-	})
-
-	t.Run("RedactedURL", func(t *testing.T) {
-		cmd := migrateCmd()
-		cmd.AddCommand(migrateApplyCmd())
-		_, err := runCmd(
-			cmd, "apply",
-			"-c", "file://"+path,
-			"--env", "local",
-			"--url", "    sqlite://my:pass@file.db",
-			"--var", "cloud_url="+srv.URL,
-		)
-		// Open errors are reported without URLs.
-		require.EqualError(t, err, `sql/sqlclient: parse open url: first path segment in URL cannot contain colon`)
-		require.Equal(t, `Error: sql/sqlclient: parse open url: first path segment in URL cannot contain colon`, *reports.Error)
-	})
-
-	t.Run("LocalDirectory", func(t *testing.T) {
-		report.DirName = "NotReported"
-		cmd := migrateCmd()
-		cmd.AddCommand(migrateApplyCmd())
-		_, err := runCmd(
-			cmd, "apply",
-			"-c", "file://"+path,
-			"--env", "local",
-			"--dir", "file://"+t.TempDir(),
-			"--url", openSQLite(t, ""),
-			"--var", "cloud_url="+srv.URL,
-		)
-		require.NoError(t, err)
-		require.Equal(t, "NotReported", report.DirName)
-	})
-}
-
-func TestMigrate_ApplyCloudReportSet(t *testing.T) {
-	var (
-		dir    migrate.MemDir
-		status int
-		report cloudapi.ReportMigrationSetInput
-		srv    = httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
-			var m struct {
-				Query     string `json:"query"`
-				Variables struct {
-					Input json.RawMessage `json:"input"`
-				} `json:"variables"`
-			}
-			require.NoError(t, json.NewDecoder(r.Body).Decode(&m))
-			switch {
-			case strings.Contains(m.Query, "query") && strings.Contains(m.Query, "me"):
-				fmt.Fprintln(w, `{"data": {"me":{ "name": "a8m", "org": "graph"}}}`)
-			case strings.Contains(m.Query, "query"):
-				// Checksum before archiving.
-				hf, err := dir.Checksum()
-				require.NoError(t, err)
-				ht, err := hf.MarshalText()
-				require.NoError(t, err)
-				require.NoError(t, dir.WriteFile(migrate.HashFileName, ht))
-				// Archive and send.
-				arc, err := migrate.ArchiveDir(&dir)
-				require.NoError(t, err)
-				fmt.Fprintf(w, `{"data":{"dirState":{"content":%q}}}`, base64.StdEncoding.EncodeToString(arc))
-			case strings.Contains(m.Query, "mutation"):
-				if status != 0 {
-					w.WriteHeader(status)
-				} else {
-					fmt.Fprint(w, `{"data":{"reportMigrationSet":{"url":"https://gh.atlasgo.cloud/deployments/sets/94489280524"}}}`)
-				}
-				require.NoError(t, json.Unmarshal(m.Variables.Input, &report))
-			default:
-				t.Fatalf("unexpected query: %s", m.Query)
-			}
-		}))
-		h = `
-variable "cloud_url" {
-  type = string
-}
-
-atlas {
-  cloud {
-    token   = "token"
-    url     = var.cloud_url
-    project = "example"
-  }
-}
-
-data "remote_dir" "migration_set" {
-  name = "migration_set"
-}
-
-variable "urls" {
-  type    = list(string)
-  default = []
-}
-
-env {
-  name = atlas.env
-  for_each = toset(var.urls)
-  url = each.value
-  migration {
-    dir = data.remote_dir.migration_set.url
-  }
-}
-`
-		p    = t.TempDir()
-		path = filepath.Join(p, "atlas.hcl")
-	)
-	t.Cleanup(srv.Close)
-	require.NoError(t, os.WriteFile(path, []byte(h), 0600))
-
-	t.Run("MultipleEnvs", func(t *testing.T) {
-		cmd := migrateCmd()
-		cmd.AddCommand(migrateApplyCmd())
-		s, err := runCmd(
-			cmd, "apply",
-			"-c", "file://"+path,
-			"--env", "local",
-			"--var", fmt.Sprintf("urls=%s", openSQLite(t, "")),
-			"--var", fmt.Sprintf("urls=%s", openSQLite(t, "")),
-			"--var", "cloud_url="+srv.URL,
-		)
-		require.NoError(t, err)
-		require.Equal(t, "No migration files to execute\nNo migration files to execute\nhttps://gh.atlasgo.cloud/deployments/sets/94489280524\n", s)
-		require.NotEmpty(t, report.ID)
-		_, err = uuid.Parse(report.ID)
-		require.NoError(t, err, "set id is not a valid uuid")
-		require.False(t, report.StartTime.IsZero())
-		require.False(t, report.EndTime.IsZero())
-		require.Equal(t, 2, report.Planned)
-		require.Len(t, report.Completed, 2)
-		require.Nil(t, report.Error)
-
-		// Verify summary log.
-		require.Len(t, report.Log, 3)
-		top := report.Log[0]
-		require.Equal(t, top.Text, "Start migration for 2 targets")
-		require.Contains(t, top.Log[0].Text, "sqlite://file")
-		require.Contains(t, top.Log[1].Text, "sqlite://file")
-		require.Equal(t, report.Log[1].Text, "Run migration: 1")
-		require.Contains(t, report.Log[1].Log[0].Text, "Target URL: sqlite://file")
-		require.Contains(t, report.Log[1].Log[1].Text, "Migration directory: mem://migration_set")
-		require.Equal(t, report.Log[2].Text, "Run migration: 2")
-		require.Contains(t, report.Log[2].Log[0].Text, "Target URL: sqlite://file")
-		require.Contains(t, report.Log[2].Log[1].Text, "Migration directory: mem://migration_set")
-	})
-
-	t.Run("WithContext", func(t *testing.T) {
-		cmd := migrateCmd()
-		cmd.AddCommand(migrateApplyCmd())
-		s, err := runCmd(
-			cmd, "apply",
-			"-c", "file://"+path,
-			"--env", "local",
-			"--context", `{ "triggerType": "KUBERNETES", "triggerVersion": "v1.2.3" }`,
-			"--var", fmt.Sprintf("urls=%s", openSQLite(t, "")),
-			"--var", fmt.Sprintf("urls=%s", openSQLite(t, "")),
-			"--var", "cloud_url="+srv.URL,
-		)
-		require.NoError(t, err)
-		require.Equal(t, "No migration files to execute\nNo migration files to execute\nhttps://gh.atlasgo.cloud/deployments/sets/94489280524\n", s)
-		require.NotNil(t, report.Context)
-		require.Equal(t, report.Context.TriggerType, "KUBERNETES")
-		require.Equal(t, report.Context.TriggerVersion, "v1.2.3")
-	})
-
-	t.Run("Error", func(t *testing.T) {
-		require.NoError(t, dir.WriteFile("1.sql", []byte("create table foo (id int)")))
-		require.NoError(t, dir.WriteFile("2.sql", []byte("create table bar (id int)")))
-
-		cmd := migrateCmd()
-		cmd.AddCommand(migrateApplyCmd())
-		s, err := runCmd(
-			cmd, "apply",
-			"-c", "file://"+path,
-			"--env", "local",
-			"--var", fmt.Sprintf("urls=%s", openSQLite(t, "create table bar (id int)")),
-			"--var", fmt.Sprintf("urls=%s", openSQLite(t, "create table bar (id int)")),
-			"--var", "cloud_url="+srv.URL,
-			"--allow-dirty",
-		)
-		require.EqualError(t, err, `sql/migrate: executing statement "create table bar (id int)" from version "2": table bar already exists`)
-		require.Contains(t, s, "Migrating to version 2 (2 migrations in total):")
-		require.Contains(t, s, "Error: sql/migrate:")
-		require.NotEmpty(t, report.ID)
-		_, err = uuid.Parse(report.ID)
-		require.NoError(t, err, "set id is not a valid uuid")
-		require.False(t, report.StartTime.IsZero())
-		require.False(t, report.EndTime.IsZero())
-		require.Equal(t, 2, report.Planned)
-		require.Len(t, report.Completed, 1)
-		require.NotNil(t, report.Error)
-		require.Equal(t, `Error: sql/migrate: executing statement "create table bar (id int)" from version "2": table bar already exists`, *report.Error)
-
-		// Verify summary log.
-		require.Len(t, report.Log, 2)
-		top := report.Log[0]
-		require.Equal(t, top.Text, "Start migration for 2 targets")
-		require.Contains(t, top.Log[0].Text, "sqlite://file")
-		require.Contains(t, top.Log[1].Text, "sqlite://file")
-		require.Equal(t, report.Log[1].Text, "Run migration: 1")
-		require.True(t, report.Log[1].Error)
-		require.Contains(t, report.Log[1].Log[0].Text, "Target URL: sqlite://file")
-		require.Contains(t, report.Log[1].Log[1].Text, "Migration directory: mem://migration_set")
-		require.Equal(t, `Error: sql/migrate: executing statement "create table bar (id int)" from version "2": table bar already exists`, report.Log[1].Log[2].Text)
-	})
-}
-
 func TestMigrate_Diff(t *testing.T) {
 	p := t.TempDir()
 	to := hclURL(t)
@@ -1891,11 +1470,11 @@ func TestMigrate_Validate(t *testing.T) {
 	s, err = runCmd(migrateValidateCmd(), "--dir", "file://"+p)
 	csErr := &migrate.ChecksumError{}
 	require.ErrorAs(t, err, &csErr)
-	require.Equal(t, 2, csErr.Line)
+	require.Equal(t, 3, csErr.Line)
 	require.Equal(t, "2_second.sql", csErr.File)
 	require.Equal(t, migrate.ReasonAdded, csErr.Reason)
 	require.Contains(t, s, "You have a checksum error")
-	require.Contains(t, s, "L 2: 2_second.sql was added")
+	require.Contains(t, s, "L3: 2_second.sql was added")
 }
 
 func TestMigrate_Hash(t *testing.T) {
