@@ -48,6 +48,9 @@ type (
 		// resources on inspection.
 		Exclude []string `spec:"exclude"`
 
+		// Schema containing the schema configuration of the env.
+		Schema *Schema `spec:"schema"`
+
 		// Migration containing the migration configuration of the env.
 		Migration *Migration `spec:"migration"`
 
@@ -60,6 +63,9 @@ type (
 		// Format of the environment.
 		Format Format `spec:"format"`
 
+		// Test configuration of the environment.
+		Test *Test `spec:"test"`
+
 		schemahcl.DefaultExtension
 		cloud  *cmdext.AtlasConfig
 		config *Project
@@ -73,6 +79,19 @@ type (
 		ExecOrder       string `spec:"exec_order"`
 		LockTimeout     string `spec:"lock_timeout"`
 		RevisionsSchema string `spec:"revisions_schema"`
+		Repo            *Repo  `spec:"repo"`
+	}
+
+	// Schema represents a schema in the registry.
+	Schema struct {
+		Src  string `spec:"src"` // Source of the schema. Typically, a URL to a computed one.
+		Repo *Repo  `spec:"repo"`
+	}
+
+	// Repo represents a repository in the schema registry
+	// for a schema or migrations directory.
+	Repo struct {
+		Name string `spec:"name"` // Name of the repository.
 	}
 
 	// Lint represents the configuration of migration linting.
@@ -99,6 +118,20 @@ type (
 		schemahcl.DefaultExtension
 	}
 
+	// Test represents the test configuration of a project or environment.
+	Test struct {
+		// Schema represents the 'schema test' configuration.
+		Schema struct {
+			Src  []string `spec:"src"`
+			Vars Vars     `spec:"vars"`
+		} `spec:"schema"`
+		// Migrate represents the 'migrate test' configuration.
+		Migrate struct {
+			Src  []string `spec:"src"`
+			Vars Vars     `spec:"vars"`
+		} `spec:"migrate"`
+	}
+
 	// SkipChanges represents the skip changes policy.
 	SkipChanges struct {
 		AddSchema        bool `spec:"add_schema"`
@@ -164,6 +197,22 @@ type (
 // envScheme defines the scheme that can be used to reference env attributes.
 const envAttrScheme = "env"
 
+// MigrationRepo returns the migration repository name, if set.
+func (e *Env) MigrationRepo() (s string) {
+	if e != nil && e.Migration != nil && e.Migration.Repo != nil {
+		s = e.Migration.Repo.Name
+	}
+	return
+}
+
+// SchemaRepo returns the desired schema repository name, if set.
+func (e *Env) SchemaRepo() (s string) {
+	if e != nil && e.Schema != nil && e.Schema.Repo != nil {
+		s = e.Schema.Repo.Name
+	}
+	return
+}
+
 // VarFromURL returns the string variable (env attribute) from the URL.
 func (e *Env) VarFromURL(s string) (string, error) {
 	u, err := url.Parse(s)
@@ -238,7 +287,9 @@ func (l *Lint) Extend(global *Lint) *Lint {
 	if l.Format == "" {
 		l.Format = global.Format
 	}
-	l.Extra = global.Extra
+	if len(l.Extra.Children) == 0 && len(l.Extra.Attrs) == 0 {
+		l.Extra = global.Extra
+	}
 	switch {
 	// Changes detector was configured on the env.
 	case l.Git.Dir != "" && l.Git.Base != "" || l.Latest != 0:
@@ -345,8 +396,13 @@ func (e *Env) DiffOptions() []schema.DiffOption {
 	return e.Diff.Options()
 }
 
-// Sources returns the paths containing the Atlas schema.
+// Sources returns the paths containing the Atlas desired schema.
+// The "src" attribute predates the "schema" block. If the "schema"
+// is defined, it takes precedence over the "src" attribute.
 func (e *Env) Sources() ([]string, error) {
+	if e.Schema != nil && e.Schema.Src != "" {
+		return []string{e.Schema.Src}, nil
+	}
 	attr, exists := e.Attr("src")
 	if !exists {
 		return nil, nil
@@ -381,6 +437,30 @@ func (e *Env) asMap() (map[string]string, error) {
 	return m, nil
 }
 
+// Extend allows extending environment blocks with
+// a global one. For example:
+//
+//	test {
+//	  schema {
+//	    src = [...]
+//	  }
+//	}
+//
+//	env "local" {
+//	  ...
+//	  test {
+//	    schema {
+//	      src = [...]
+//	    }
+//	  }
+//	}
+func (t *Test) Extend(global *Test) *Test {
+	if t == nil {
+		return global
+	}
+	return t
+}
+
 // EnvByName parses and returns the project configuration with selected environments.
 func EnvByName(cmd *cobra.Command, name string, vars map[string]cty.Value) (*Project, []*Env, error) {
 	envs := make(map[string][]*Env)
@@ -434,6 +514,7 @@ func EnvByName(cmd *cobra.Command, name string, vars map[string]cty.Value) (*Pro
 		if err := e.Lint.remainedLog(); err != nil {
 			return nil, nil, err
 		}
+		e.Test = e.Test.Extend(project.Test)
 		envs[e.Name] = append(envs[e.Name], e)
 	}
 	envsCache.store(GlobalFlags.ConfigURL, name, vars, project, envs[name])
@@ -524,19 +605,9 @@ func parseConfig(ctx context.Context, path, env string, vars map[string]cty.Valu
 				}),
 			}),
 			schemahcl.WithFunctions(map[string]function.Function{
-				"file": schemahcl.MakeFileFunc(base),
-				"getenv": function.New(&function.Spec{
-					Params: []function.Parameter{
-						{
-							Name: "key",
-							Type: cty.String,
-						},
-					},
-					Type: function.StaticReturnType(cty.String),
-					Impl: func(args []cty.Value, retType cty.Type) (cty.Value, error) {
-						return cty.StringVal(os.Getenv(args[0].AsString())), nil
-					},
-				}),
+				"file":   schemahcl.MakeFileFunc(base),
+				"glob":   schemahcl.MakeGlobFunc(base),
+				"getenv": getEnvFunc,
 			}),
 		)...,
 	)
@@ -605,3 +676,18 @@ const (
 )
 
 var ReviewModes = []string{ReviewAlways, ReviewWarning, ReviewError}
+
+// getEnvFunc is a custom HCL function that returns
+// the value of an environment variable.
+var getEnvFunc = function.New(&function.Spec{
+	Params: []function.Parameter{
+		{
+			Name: "key",
+			Type: cty.String,
+		},
+	},
+	Type: function.StaticReturnType(cty.String),
+	Impl: func(args []cty.Value, retType cty.Type) (cty.Value, error) {
+		return cty.StringVal(os.Getenv(args[0].AsString())), nil
+	},
+})
