@@ -12,12 +12,13 @@ import (
 	"net/url"
 	"os"
 	"path/filepath"
+	"strings"
 
-	"ariga.io/atlas/cmd/atlas/internal/cmdlog"
 	cmdmigrate "ariga.io/atlas/cmd/atlas/internal/migrate"
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/schema"
 	"ariga.io/atlas/sql/sqlclient"
+
 	"github.com/hashicorp/hcl/v2"
 	"github.com/hashicorp/hcl/v2/hclparse"
 	"github.com/hashicorp/hcl/v2/hclsyntax"
@@ -43,10 +44,11 @@ type (
 )
 
 // Close redirects calls to Close to the enclosed io.Closer.
-func (r *StateReadCloser) Close() {
+func (r *StateReadCloser) Close() error {
 	if r.Closer != nil {
-		r.Closer.Close()
+		return r.Closer.Close()
 	}
+	return nil
 }
 
 // StateReaderSQL returns a migrate.StateReader from an SQL file or a directory of migrations.
@@ -120,16 +122,30 @@ func stateSchemaSQL(ctx context.Context, cfg *StateReaderConfig, dir migrate.Dir
 	if cfg.Dev == nil {
 		return nil, errNoDevURL
 	}
-	log := cmdlog.NewMigrateApply(ctx, cfg.Dev, dir)
+	log := &errorRecorder{}
 	r, err := stateReaderSQL(ctx, cfg, dir, []migrate.ExecutorOption{migrate.WithLogger(log)}, nil)
-	if n := len(log.Applied); err != nil && n > 0 {
-		if serr := log.Applied[n-1].Error; serr != nil && serr.Stmt != "" && serr.Text != "" {
-			err = fmt.Errorf("read state from %q: executing statement: %q: %s", log.Applied[n-1].Name(), serr.Stmt, serr.Text)
-		}
+	if n := len(log.applied); err != nil && n > 0 && log.stmt != "" && log.text != "" {
+		err = fmt.Errorf("read state from %q: executing statement: %q: %s", log.applied[n-1], log.stmt, log.text)
 	}
 	return r, err
 }
 
+type errorRecorder struct {
+	applied    []string // applied files.
+	stmt, text string   // error statement and text.
+}
+
+// Log implements migrate.Logger.
+func (r *errorRecorder) Log(e migrate.LogEntry) {
+	switch e := e.(type) {
+	case migrate.LogFile:
+		r.applied = append(r.applied, e.File.Name())
+	case migrate.LogError:
+		r.stmt = e.SQL
+		r.text = e.Error.Error()
+	}
+}
+
 // stateReaderSQL returns a migrate.StateReader from an SQL file or a directory of migrations.
 func stateReaderSQL(ctx context.Context, cfg *StateReaderConfig, dir migrate.Dir, optsExec []migrate.ExecutorOption, optsReplay []migrate.ReplayOption) (*StateReadCloser, error) {
 	if cfg.Dev == nil {
@@ -206,9 +222,18 @@ func stateReaderHCL(_ context.Context, config *StateReaderConfig, paths []string
 			config.Dev.URL.Schema,
 		)
 	}
-	var normalized bool
+	var (
+		normalized  bool
+		schemaScope string
+	)
+	// The "Schema" below indicates the HCL represents a single
+	// database schema, and the work is scoped to this schema.
+	if len(realm.Schemas) == 1 && (config.Dev != nil && config.Dev.URL.Schema != "" || config.Client != nil && config.Client.URL.Schema != "") {
+		schemaScope = realm.Schemas[0].Name
+	}
 	return &StateReadCloser{
-		HCL: true,
+		HCL:    true,
+		Schema: schemaScope,
 		// Defer normalization until the first call to ReadState. This is required because the same
 		// dev-database is used for both migration replaying and schema normalization. As a result,
 		// objects created by the migrations, which are not yet supported by Atlas, such as functions,
@@ -329,14 +354,15 @@ const (
 
 // File extensions supported by the file driver.
 const (
-	FileTypeHCL = ".hcl"
-	FileTypeSQL = ".sql"
+	FileTypeHCL  = ".hcl"
+	FileTypeSQL  = ".sql"
+	FileTypeTest = ".test.hcl"
 )
 
 // mayParse will parse the file in path if it is an HCL file. If the file is an Atlas
 // project file an error is returned.
 func mayParse(p *hclparse.Parser, path string) error {
-	if n := filepath.Base(path); filepath.Ext(n) != FileTypeHCL {
+	if n := filepath.Base(path); filepath.Ext(n) != FileTypeHCL && !strings.HasSuffix(path, FileTypeTest) {
 		return nil
 	}
 	switch f, diag := p.ParseHCLFile(path); {
