@@ -8,13 +8,21 @@ import (
 	"bytes"
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
+	"io"
+	"net/url"
+	"reflect"
 	"slices"
 	"sort"
+	"strconv"
 	"strings"
+	"sync"
 	"text/template"
 	"time"
 
+	cmdmigrate "ariga.io/atlas/cmd/atlas/internal/migrate"
+	"ariga.io/atlas/cmd/atlas/internal/migrate/ent/revision"
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/schema"
 	"ariga.io/atlas/sql/sqlclient"
@@ -24,6 +32,7 @@ import (
 
 var (
 	ColorCyan         = color.CyanString
+	ColorGray         = color.New(color.Attribute(90))
 	ColorGreen        = color.HiGreenString
 	ColorRed          = color.HiRedString
 	ColorRedBgWhiteFg = color.New(color.FgHiWhite, color.BgHiRed).SprintFunc()
@@ -102,13 +111,13 @@ func (f Files) MarshalJSON() ([]byte, error) {
 }
 
 // NewEnv returns an initialized Env.
-func NewEnv(c *sqlclient.Client, dir migrate.Dir) Env {
+func NewEnv(c *sqlclient.Client, dirURL *url.URL) Env {
 	e := Env{
 		Driver: c.Name,
 		URL:    c.URL,
 	}
-	if p, ok := dir.(interface{ Path() string }); ok {
-		e.Dir = p.Path()
+	if dirURL != nil {
+		e.Dir = dirURL.Redacted()
 	}
 	return e
 }
@@ -185,6 +194,111 @@ func (r *MigrateStatus) FromCheckpoint() bool {
 	return ok && ck.IsCheckpoint()
 }
 
+// StatusReporter is used to gather information about migration status.
+type StatusReporter struct {
+	// Client configures the connection to the database to file a MigrateStatus for.
+	Client *sqlclient.Client
+	// DirURL of the migration directory.
+	DirURL *url.URL
+	// Dir is used for scanning and validating the migration directory.
+	Dir migrate.Dir
+	// Schema name the revision table resides in.
+	Schema string
+}
+
+// Report creates and writes a MigrateStatus.
+func (r *StatusReporter) Report(ctx context.Context) (*MigrateStatus, error) {
+	rep := &MigrateStatus{Env: NewEnv(r.Client, r.DirURL)}
+	// Check if there already is a revision table in the defined schema.
+	// Inspect schema and check if the table does already exist.
+	sch, err := r.Client.InspectSchema(ctx, r.Schema, &schema.InspectOptions{Tables: []string{revision.Table}})
+	if err != nil && !schema.IsNotExistError(err) {
+		return nil, err
+	}
+	if schema.IsNotExistError(err) || func() bool { _, ok := sch.Table(revision.Table); return !ok }() {
+		// Either schema or table does not exist.
+		if rep.Available, err = migrate.FilesFromLastCheckpoint(r.Dir); err != nil {
+			return nil, err
+		}
+		rep.Pending = rep.Available
+	} else {
+		// Both exist, fetch their data.
+		rrw, err := cmdmigrate.RevisionsForClient(ctx, r.Client, r.Schema)
+		if err != nil {
+			return nil, err
+		}
+		if err := rrw.Migrate(ctx); err != nil {
+			return nil, err
+		}
+		ex, err := migrate.NewExecutor(r.Client.Driver, r.Dir, rrw)
+		if err != nil {
+			return nil, err
+		}
+		rep.Applied, err = rrw.ReadRevisions(ctx)
+		if err != nil {
+			return nil, err
+		}
+		if rep.Pending, err = ex.Pending(ctx); err != nil && !errors.Is(err, migrate.ErrNoPendingFiles) {
+			if err1 := (*migrate.HistoryNonLinearError)(nil); errors.As(err, &err1) {
+				rep.Error = err1.Error()
+				rep.Status = statusPending
+				rep.Pending = err1.Pending
+				rep.OutOfOrder = err1.OutOfOrder
+				// Non-linear error means at least one file was applied.
+				rep.Current = rep.Applied[len(rep.Applied)-1].Version
+				return rep, nil
+			}
+			return nil, err
+		}
+		// If no files were applied, all pending files are
+		// available. The first one might be a checkpoint.
+		if len(rep.Applied) == 0 {
+			rep.Available = rep.Pending
+		} else if rep.Available, err = r.Dir.Files(); err != nil {
+			return nil, err
+		}
+	}
+	switch len(rep.Pending) {
+	case len(rep.Available):
+		rep.Current = "No migration applied yet"
+	default:
+		rep.Current = rep.Applied[len(rep.Applied)-1].Version
+	}
+	if len(rep.Pending) == 0 {
+		rep.Status = statusOK
+		rep.Next = "Already at latest version"
+	} else {
+		rep.Status = statusPending
+		rep.Next = rep.Pending[0].Version()
+	}
+	// If the last one is partially applied (and not manually resolved).
+	if len(rep.Applied) != 0 {
+		last := rep.Applied[len(rep.Applied)-1]
+		if !last.Type.Has(migrate.RevisionTypeResolved) && last.Applied < last.Total {
+			rep.SQL = strings.ReplaceAll(last.ErrorStmt, "\n", " ")
+			rep.Error = strings.ReplaceAll(last.Error, "\n", " ")
+			rep.Count = last.Applied
+			idx := migrate.FilesLastIndex(rep.Available, func(f migrate.File) bool {
+				return f.Version() == last.Version
+			})
+			if idx == -1 {
+				return nil, fmt.Errorf("migration file with version %q not found", last.Version)
+			}
+			stmts, err := migrate.FileStmts(r.Client.Driver, rep.Available[idx])
+			if err != nil {
+				return nil, err
+			}
+			rep.Total = len(stmts)
+		}
+	}
+	return rep, nil
+}
+
+const (
+	statusOK      = "OK"
+	statusPending = "PENDING"
+)
+
 // MigrateSetTemplate holds the default template of the 'migrate set' command.
 var MigrateSetTemplate = template.Must(template.New("set").
 	Funcs(ColorTemplateFuncs).Parse(`
@@ -287,8 +401,8 @@ var (
 				Parse(`{{- if not .Pending -}}
 {{- println "No migration files to execute" }}
 {{- else -}}
-Migrating to version {{ cyan .Target }}{{ with .Current }} from {{ cyan . }}{{ end }} ({{ len .Pending }} migrations in total):
-{{ range $i, $f := .Applied }}
+{{- println .Header }}
+{{- range $i, $f := .Applied }}
 	{{- println }}
 	{{- $checkFailed := false }}
 	{{- range $cf := $f.Checks }}
@@ -319,6 +433,11 @@ Migrating to version {{ cyan .Target }}{{ with .Current }} from {{ cyan . }}{{ e
 	{{- else }}
 		{{- printf "  %s ok (%s)\n" (yellow "--") (yellow (.End.Sub .Start).String) }}
 	{{- end }}
+{{- else }}
+	{{- println }}
+	{{- with .Error }}
+		{{- println "   " (redBgWhiteFg .) }}
+	{{- end }}
 {{- end }}
 {{- println }}
 {{- println " " (cyan "-------------------------") }}
@@ -356,10 +475,10 @@ type (
 )
 
 // NewMigrateApply returns an MigrateApply.
-func NewMigrateApply(ctx context.Context, client *sqlclient.Client, dir migrate.Dir) *MigrateApply {
+func NewMigrateApply(ctx context.Context, client *sqlclient.Client, dirURL *url.URL) *MigrateApply {
 	return &MigrateApply{
 		ctx:   ctx,
-		Env:   NewEnv(client, dir),
+		Env:   NewEnv(client, dirURL),
 		Start: time.Now(),
 	}
 }
@@ -418,6 +537,7 @@ func (a *MigrateApply) Log(e migrate.LogEntry) {
 		f := a.Applied[len(a.Applied)-1]
 		f.Applied = append(f.Applied, e.SQL)
 	case migrate.LogError:
+		// Error during migration.
 		if l := len(a.Applied); l > 0 {
 			f := a.Applied[len(a.Applied)-1]
 			f.End = time.Now()
@@ -426,6 +546,11 @@ func (a *MigrateApply) Log(e migrate.LogEntry) {
 				Stmt: e.SQL,
 				Text: e.Error.Error(),
 			}
+			// Error during pre stages, such as
+			// scanning migration statements.
+		} else {
+			a.End = time.Now()
+			a.Error = e.Error.Error()
 		}
 	case migrate.LogDone:
 		n := time.Now()
@@ -436,7 +561,25 @@ func (a *MigrateApply) Log(e migrate.LogEntry) {
 	}
 }
 
-// Summary returns a footer of the migration attempt.
+// Header returns a header of the migration log.
+func (a *MigrateApply) Header() string {
+	if len(a.Pending) == 0 {
+		return "No migration files to execute"
+	}
+	var b strings.Builder
+	b.WriteString("Migrating to version ")
+	b.WriteString(ColorCyan(a.Target))
+	if a.Current != "" {
+		b.WriteString(" from ")
+		b.WriteString(ColorCyan(a.Current))
+	}
+	b.WriteString(" (")
+	b.WriteString(strconv.Itoa(len(a.Pending)))
+	b.WriteString(" migrations in total):")
+	return b.String()
+}
+
+// Summary returns a footer of the migration log.
 func (a *MigrateApply) Summary(ident string) string {
 	var (
 		passedC, failedC int
@@ -800,38 +943,7 @@ func (s *SchemaInspect) MarshalSQL(indent ...string) (string, error) {
 }
 
 func sqlInspect(report *SchemaInspect, indent ...string) (string, error) {
-	var changes schema.Changes
-	for _, o := range report.Realm.Objects {
-		changes = append(changes, &schema.AddObject{O: o})
-	}
-	for _, s := range report.Realm.Schemas {
-		// Generate commands for creating the schemas on realm-mode.
-		if report.client.URL.Schema == "" {
-			changes = append(changes, &schema.AddSchema{S: s})
-		}
-		for _, o := range s.Objects {
-			changes = append(changes, &schema.AddObject{O: o})
-		}
-		for _, t := range s.Tables {
-			changes = append(changes, &schema.AddTable{T: t})
-			for _, r := range t.Triggers {
-				changes = append(changes, &schema.AddTrigger{T: r})
-			}
-		}
-		for _, v := range s.Views {
-			changes = append(changes, &schema.AddView{V: v})
-			for _, r := range v.Triggers {
-				changes = append(changes, &schema.AddTrigger{T: r})
-			}
-		}
-		for _, f := range s.Funcs {
-			changes = append(changes, &schema.AddFunc{F: f})
-		}
-		for _, p := range s.Procs {
-			changes = append(changes, &schema.AddProc{P: p})
-		}
-	}
-	return fmtPlan(report.ctx, report.client, changes, indent)
+	return fmtPlan(report.ctx, report.client, cmdmigrate.ChangesToRealm(report.client, report.Realm), indent)
 }
 
 // SchemaDiff contains a summary of the 'schema diff' command.
@@ -882,13 +994,13 @@ func sqlDiff(diff *SchemaDiff, indent ...string) (string, error) {
 	return fmtPlan(diff.ctx, diff.client, diff.Changes, indent)
 }
 
-func fmtPlan(ctx context.Context, client *sqlclient.Client, changes schema.Changes, indent []string) (string, error) {
+func fmtPlan(ctx context.Context, client *sqlclient.Client, changes schema.Changes, indent []string, edit ...func(*migrate.Plan)) (string, error) {
 	if len(indent) > 1 {
 		return "", fmt.Errorf("unexpected number of arguments: %d", len(indent))
 	}
 	plan, err := client.PlanChanges(ctx, "plan", changes, func(o *migrate.PlanOptions) {
 		o.Mode = migrate.PlanModeDump
-		// Disable tables qualifier in schema-mode.
+		// Disable object qualifier in schema-mode.
 		if client.URL.Schema != "" {
 			o.SchemaQualifier = new(string)
 		}
@@ -899,14 +1011,15 @@ func fmtPlan(ctx context.Context, client *sqlclient.Client, changes schema.Chang
 	if err != nil {
 		return "", err
 	}
-	switch files, err := migrate.DefaultFormatter.Format(plan); {
-	case err != nil:
-		return "", err
-	case len(files) != 1:
-		return "", fmt.Errorf("unexpected number of files: %d", len(files))
-	default:
-		return string(files[0].Bytes()), nil
+	// Optional edit functions.
+	for i := range edit {
+		edit[i](plan)
 	}
+	f, err := migrate.DefaultFormatter.FormatFile(plan)
+	if err != nil {
+		return "", err
+	}
+	return string(f.Bytes()), nil
 }
 
 func mermaid(i *SchemaInspect, _ ...string) (string, error) {
@@ -1030,3 +1143,20 @@ func indentLn(input string, indent int) string {
 	pad := strings.Repeat(" ", indent)
 	return strings.ReplaceAll(input, "\n", "\n"+pad)
 }
+
+// WarnOnce allow writing warning messages to the given writer,
+// but ensures only one message will be written in process run.
+func WarnOnce(w io.Writer, text string) error {
+	if !reflect.TypeOf(w).Comparable() {
+		_, err := w.Write([]byte(text))
+		return err
+	}
+	if _, loaded := oneWrites.LoadOrStore(w, true); !loaded {
+		_, err := w.Write([]byte(text))
+		return err
+	}
+	return nil
+}
+
+// oneWrites per writer.
+var oneWrites sync.Map
