@@ -8,6 +8,7 @@ import (
 	"bytes"
 	"context"
 	"fmt"
+
 	"os"
 	"os/signal"
 	"syscall"
@@ -15,6 +16,7 @@ import (
 
 	"ariga.io/atlas/cmd/atlas/internal/cmdapi"
 	"ariga.io/atlas/cmd/atlas/internal/cmdapi/vercheck"
+	"ariga.io/atlas/cmd/atlas/internal/cmdlog"
 	_ "ariga.io/atlas/cmd/atlas/internal/docker"
 	_ "ariga.io/atlas/sql/mysql"
 	_ "ariga.io/atlas/sql/mysql/mysqlcheck"
@@ -22,12 +24,13 @@ import (
 	_ "ariga.io/atlas/sql/postgres/postgrescheck"
 	_ "ariga.io/atlas/sql/sqlite"
 	_ "ariga.io/atlas/sql/sqlite/sqlitecheck"
-	"golang.org/x/mod/semver"
 
 	_ "github.com/go-sql-driver/mysql"
 	_ "github.com/lib/pq"
 	_ "github.com/libsql/libsql-client-go/libsql"
+	"github.com/mattn/go-isatty"
 	_ "github.com/mattn/go-sqlite3"
+	"golang.org/x/mod/semver"
 )
 
 func main() {
@@ -47,12 +50,16 @@ func main() {
 		<-stop // will not block if no signal received due to main routine exiting
 		os.Exit(1)
 	}()
-	ctx = extendContext(context.Background())
+	ctx, err := extendContext(ctx)
+	if err != nil {
+		fmt.Fprintln(os.Stderr, err)
+		os.Exit(1)
+	}
 	ctx, done := initialize(ctx)
 	update := checkForUpdate(ctx)
-	err := cmdapi.Root.ExecuteContext(ctx)
+	err = cmdapi.Root.ExecuteContext(ctx)
 	if u := update(); u != "" {
-		fmt.Fprintln(os.Stderr, u)
+		_ = cmdlog.WarnOnce(os.Stderr, cmdlog.ColorCyan(u))
 	}
 	done(err)
 	if err != nil {
@@ -68,9 +75,7 @@ const (
 
 func noText() string { return "" }
 
-// checkForUpdate checks for version updates and security advisories for Atlas.
 func checkForUpdate(ctx context.Context) func() string {
-	done := make(chan struct{})
 	version := cmdapi.Version()
 	// Users may skip update checking behavior.
 	if v := os.Getenv(envNoUpdate); v != "" {
@@ -80,20 +85,28 @@ func checkForUpdate(ctx context.Context) func() string {
 	if !semver.IsValid(version) {
 		return noText
 	}
+	endpoint := vercheckEndpoint(ctx)
+	vc := vercheck.New(endpoint)
+	if isatty.IsTerminal(os.Stdout.Fd()) {
+		return bgCheck(ctx, version, vc)
+	}
+	return func() string {
+		msg, _ := runCheck(ctx, vc, version)
+		return msg
+	}
+}
+
+// bgCheck checks for version updates and security advisories for Atlas in the background.
+func bgCheck(ctx context.Context, version string, vc *vercheck.VerChecker) func() string {
+	done := make(chan struct{})
 	var message string
 	go func() {
 		defer close(done)
-		endpoint := vercheckEndpoint(ctx)
-		vc := vercheck.New(endpoint)
-		payload, err := vc.Check(ctx, version)
+		msg, err := runCheck(ctx, vc, version)
 		if err != nil {
 			return
 		}
-		var b bytes.Buffer
-		if err := vercheck.Notify.Execute(&b, payload); err != nil {
-			return
-		}
-		message = b.String()
+		message = msg
 	}()
 	return func() string {
 		select {
@@ -103,3 +116,15 @@ func checkForUpdate(ctx context.Context) func() string {
 		return message
 	}
 }
+
+func runCheck(ctx context.Context, vc *vercheck.VerChecker, version string) (string, error) {
+	payload, err := vc.Check(ctx, version)
+	if err != nil {
+		return "", err
+	}
+	var b bytes.Buffer
+	if err := vercheck.Notify.Execute(&b, payload); err != nil {
+		return "", err
+	}
+	return b.String(), nil
+}
