@@ -1,3 +1,5 @@
+const footer = require('./src/constants/footer');
+
 /** @type {import('@docusaurus/types').DocusaurusConfig} */
 module.exports = {
   title: 'Atlas | Manage your database schema as code',
@@ -11,7 +13,7 @@ module.exports = {
   themeConfig: {
     image: "https://og.atlasgo.io/image?title=Schema%20Migration%20Tool%20for%20any%20Language",
     prism: {
-      additionalLanguages: ['hcl', 'java', 'properties', 'gradle', 'php'],
+      additionalLanguages: ['hcl', 'java', 'properties', 'gradle', 'php', 'csharp', 'powershell', 'bash'],
       magicComments: [
         {
           className: 'theme-code-block-highlighted-line',
@@ -63,7 +65,7 @@ module.exports = {
           position: 'right',
         },
         {
-          to: 'getting-started',
+          to: 'docs',
           label: 'Docs',
           position: 'left',
         },
@@ -78,78 +80,16 @@ module.exports = {
           position: 'left'
         },
         {
-          to: 'support',
-          label: 'Support',
-          position: 'left',
-        },
-        {
-          to: 'https://atlasgo.cloud/?utm_term=topnav',
-          label: 'Cloud',
+          to: 'pricing',
+          label: 'Pricing',
           position: 'left',
         },
       ],
     },
-    //
-    "footer": {
-      "links": [
-        {
-          "title": "Docs",
-          "items": [
-            {"label": "Getting Started", "to": "getting-started"},
-            {"label": "Atlas vs Others ", "to": "atlas-vs-others"},
-            {"label": "CLI Reference", "to": "cli-reference"},
-            {"label": "Blog", "to": "blog"},
-            {"label": "Guides", "to": "guides"},
-            {"label": "GoDoc", "to": "https://pkg.go.dev/ariga.io/atlas"},
-          ]
-        },
-        {
-          "title": "Community",
-          "items": [
-            {"label": "GitHub", "to": "https://github.com/ariga/atlas"},
-            {"label": "Discord", "to": "https://discord.gg/zZ6sWVg6NT"},
-            {"label": "Twitter", "to": "https://twitter.com/ariga_io"},
-            {"label": "Newsletter", "to": "https://atlasnewsletter.substack.com/"},
-            {"label": "YouTube", "to": "https://youtube.com/@ariga_io"}
-          ]
-        },
-        {
-          "title": "Integrations",
-          "items": [
-            {"label": "GitHub Actions", "to": "/integrations/github-actions"},
-            {"label": "Kubernetes Operator", "to": "/integrations/kubernetes/operator"},
-            {"label": "Terraform", "to": "/integrations/terraform-provider"},
-            {"label": "Go API", "to": "/integrations/go-api"}
-          ]
-        },
-        {
-          "title": "Atlas Cloud",
-          "items": [
-            {"label": "Discover Atlas Cloud", "to": "https://atlasgo.cloud/?utm_term=footer"},
-            {"label": "Live Demo", "to": "https://gh.atlasgo.cloud/projects?utm_term=footer"},
-            {"label": "Sign Up", "to": "https://auth.atlasgo.cloud/signup?utm_term=footer"},
-          ]
-        },
-        {
-          "title": "Legal",
-          "items": [
-            {"label": "Privacy Policy", "to": "https://ariga.io/legal/privacy"},
-            {"label": "Terms of Service", "to": "https://ariga.io/legal/tos"},
-          ]
-        },
-      ],
-      copyright: `
-      Copyright © ${new Date().getFullYear()} The Atlas Authors.
-      The Go gopher was designed by <a href="http://reneefrench.blogspot.com/">Renee French</a>.
-      <br/>
-      The design for the Go gopher is licensed under the Creative Commons 3.0 Attributions license. Read this 
-      <a href="https://blog.golang.org/gopher">article</a> for more details.
-      <br/>
-      `,
-    },
+    footer,
     announcementBar: {
-      id: 'announcementBar-13', // Increment on change
-      content: `<strong>Atlas v0.23: Redshift, CircleCI, GORM Views, and more. </strong><a class="cta" href="https://atlasgo.io/blog/2024/05/21/atlas-v-0-23">Read it here</a>`,
+      id: 'announcementBar-17', // Increment on change
+      content: `<strong>Atlas on The New Stack:</strong> Play Rock, Paper, Scissors on Postgres! <a class="cta" href="https://thenewstack.io/build-a-rock-paper-scissors-game-on-postgresql-with-database-programming/">Read it here</a>`,
       isCloseable: true,
     },
   },
@@ -183,8 +123,8 @@ module.exports = {
             from: '/dev-database',
           },
           {
-            to: '/guides/ddl',
-            from: ['/ddl/intro', '/concepts/ddl'],
+            to: '/atlas-schema/hcl',
+            from: ['/ddl/intro', '/concepts/ddl', '/guides/ddl'],
           },
           {
             to: '/atlas-schema/input-variables',
@@ -218,10 +158,6 @@ module.exports = {
             to: '/guides/postgres/serial-columns',
             from: '/knowledge/postgres/serial-columns',
           },
-          {
-            to: '/guides/ddl',
-            from: '/knowledge/ddl',
-          },
           {
             to: '/concepts/url',
             from: '/url',
@@ -246,6 +182,18 @@ module.exports = {
             from: '/ui/intro',
             to: '/cloud/getting-started',
           },
+          {
+            to: "/declarative/inspect",
+            from: "/cli/getting-started/inspection",
+          },
+          {
+            to: "/declarative/apply",
+            from: "/cli/getting-started/applying-schemas",
+          },
+          {
+            to: "/guides/deploying/remote-directories",
+            from: "/guides/deploying/from-cloud",
+          },
         ],
       },
     ],
@@ -268,6 +216,8 @@ module.exports = {
     "./plugins/utm-plugin",
     "./plugins/intercom",
     "./plugins/page-view",
+    "./plugins/tailwind",
+    "./plugins/scroll-to-top"
   ],
   presets: [
     [
