@@ -9,7 +9,10 @@ package main
 //go:generate go run . -flavor Community -suffix oss
 
 func init() {
-	data.GoVersions = goVersions{"1.21"}
+	data.GoVersions = goVersions{"1.22"}
+	data.GlobalEnv = []struct{ K, V string }{
+		{K: "ATLAS_NO_UPGRADE_SUGGESTIONS", V: "1"},
+	}
 	data.Jobs = append(jobs,
 		Job{
 			Version: "tidb5",
