@@ -195,6 +195,8 @@ var data = struct {
 	Flavor, Tags, Runner, Suffix string
 	GoVersions                   goVersions
 	Concurrency                  concurrency
+	SharedSteps                  []Step
+	GlobalEnv                    []struct{ K, V string }
 }{
 	Concurrency: concurrency{
 		group:  "${{ github.workflow }}-${{ github.head_ref || github.run_id }}",
