@@ -30,14 +30,18 @@ hd = <<-EOT
   hello
   world
 EOT
+vars = {
+  a = "a"
+}
 `
 	var test struct {
-		Int        int      `spec:"i"`
-		Bool       bool     `spec:"b"`
-		Str        string   `spec:"s"`
-		StringList []string `spec:"sl"`
-		BoolList   []bool   `spec:"bl"`
-		Heredoc    string   `spec:"hd"`
+		Int        int                  `spec:"i"`
+		Bool       bool                 `spec:"b"`
+		Str        string               `spec:"s"`
+		StringList []string             `spec:"sl"`
+		BoolList   []bool               `spec:"bl"`
+		Heredoc    string               `spec:"hd"`
+		Vars       map[string]cty.Value `spec:"vars"`
 	}
 	err := New().EvalBytes([]byte(f), &test, nil)
 	require.NoError(t, err)
@@ -47,6 +51,7 @@ EOT
 	require.EqualValues(t, []string{"hello", "world"}, test.StringList)
 	require.EqualValues(t, []bool{true, false}, test.BoolList)
 	require.EqualValues(t, "hello\nworld\n", test.Heredoc)
+	require.EqualValues(t, "a", test.Vars["a"].AsString())
 	// Heredoc needs to be explicitly formatted this way.
 	test.Heredoc = "<<-EOT\n  hello\n  world\nEOT"
 	marshal, err := Marshal(&test)
@@ -197,6 +202,27 @@ func ExampleMarshal() {
 	// }
 }
 
+func TestIgnore(t *testing.T) {
+	var test struct {
+		IgnoreAttr    string `spec:"-"`
+		NotIgnoreAttr string `spec:"not_ignore_attr"`
+		IgnoreBlock   struct {
+			Name string `spec:",name"`
+		} `spec:"-"`
+		NotIgnore struct {
+			Name string `spec:",name"`
+		} `spec:"not_ignore_block"`
+	}
+	test.IgnoreAttr, test.NotIgnoreAttr = "ignore", "not ignore"
+	test.IgnoreBlock.Name, test.NotIgnore.Name = "ignore", "not ignore"
+	buf, err := Marshal(&test)
+	require.NoError(t, err)
+	require.Equal(t, `not_ignore_attr = "not ignore"
+not_ignore_block "not ignore" {
+}
+`, string(buf))
+}
+
 func TestInterface(t *testing.T) {
 	type (
 		Animal interface {
@@ -428,6 +454,7 @@ env "prod" {
 env "staging" {
   for_each = toset(var.domains)
   url = "${each.value.name}:${each.value.port}"
+  driver = MYSQL
 }
 
 env "dev" {
@@ -440,6 +467,7 @@ env "dev" {
 `)
 	)
 	require.NoError(t, New(
+		WithScopedEnums("env.driver", "MYSQL", "POSTGRES"),
 		WithDataSource("sql", func(_ context.Context, ectx *hcl.EvalContext, b *hclsyntax.Block) (cty.Value, error) {
 			attrs, diags := b.Body.JustAttributes()
 			if diags.HasErrors() {
@@ -1072,3 +1100,99 @@ bar "b1" {
 		Ref:  &Ref{V: "$foo.f1"},
 	}, doc.Bar[0])
 }
+
+func Test_MarshalAttr(t *testing.T) {
+	var doc struct {
+		DefaultExtension
+	}
+	doc.Extra.Attrs = append(
+		doc.Extra.Attrs,
+		StringEnumsAttr("mixed1", &EnumString{S: "string"}, &EnumString{E: "enum"}),
+		StringEnumsAttr("mixed2", &EnumString{E: "enum1"}, &EnumString{E: "enum2"}),
+		StringEnumsAttr("mixed3", &EnumString{S: "string1"}, &EnumString{S: "string1"}),
+	)
+	buf, err := Marshal(&doc)
+	require.NoError(t, err)
+	require.Equal(t, `mixed1 = ["string", enum]
+mixed2 = [enum1, enum2]
+mixed3 = ["string1", "string1"]
+`, string(buf))
+}
+
+func Test_WithPos(t *testing.T) {
+	var (
+		doc struct {
+			Bar struct {
+				A int `spec:"a"`
+				DefaultExtension
+			} `spec:"bar"`
+			DefaultExtension
+		}
+		b = []byte(`
+foo {}
+foo {
+  bar {
+    baz = 1
+  }
+}
+baz = 1
+bar {
+  a = 1
+}
+`)
+	)
+	require.NoError(t, New(WithPos()).EvalBytes(b, &doc, nil))
+	at, ok := doc.Extra.Attr("baz")
+	require.True(t, ok)
+	require.Equal(t, 8, at.Range().Start.Line)
+	rs := doc.Extra.Resources("foo")
+	require.Len(t, rs, 2)
+	require.Equal(t, 2, rs[0].Range().Start.Line)
+	require.Equal(t, 3, rs[1].Range().Start.Line)
+	require.Equal(t, 4, rs[1].Children[0].Range().Start.Line)
+	require.Equal(t, 5, rs[1].Children[0].Attrs[0].Range().Start.Line)
+	require.NotNil(t, doc.Bar.Extra.Range(), "position should be attached to the resource")
+}
+
+func TestExtendedBlockDef(t *testing.T) {
+	var (
+		doc struct {
+			DefaultExtension
+		}
+		b = []byte(`
+schema "public" {}
+table "users" {}
+materialized "users_view2" {
+  schema = schema.public
+  as = "SELECT * FROM script_matview_inspect.users"
+  column "id" {
+    null = false
+  }
+  column "a" {
+    null = false
+  }
+  column "b" {
+    null = false
+  }
+  primary_key {
+    columns = [column.id]
+  }
+  populate = true
+}
+materialized "users_view" {
+  schema = schema.public
+  to = table.users
+  as = "SELECT * FROM script_matview_inspect.users"
+  index "i" {
+    on {
+      expr = "a"
+    }
+  }
+  primary_key {
+    using = index.i # Not a real syntax.
+  }
+}
+`)
+	)
+	require.NoError(t, New().EvalBytes(b, &doc, nil))
+}
