@@ -7,11 +7,13 @@ package migrate_test
 import (
 	"archive/tar"
 	"bytes"
+	"compress/gzip"
 	_ "embed"
 	"io"
 	"os"
 	"path/filepath"
 	"testing"
+	"text/template"
 	"time"
 
 	"ariga.io/atlas/sql/migrate"
@@ -91,7 +93,7 @@ func TestValidate(t *testing.T) {
 	d, err := migrate.NewLocalDir(p)
 	require.NoError(t, err)
 	require.NoError(t, d.WriteFile("atlas.sum", hash))
-	require.Equal(t, removed(1, 2, 48, "1_initial.down.sql"), migrate.Validate(d))
+	require.Equal(t, removed(2, 2, 48, "1_initial.down.sql"), migrate.Validate(d))
 
 	td := "testdata/migrate"
 	d, err = migrate.NewLocalDir(td)
@@ -128,13 +130,13 @@ func TestValidate(t *testing.T) {
 	t.Cleanup(func() {
 		require.NoError(t, os.WriteFile(filepath.Join(td, "1_initial.up.sql"), initialUp, 0644))
 	})
-	require.Equal(t, edited(2, 2, 115, "1_initial.up.sql"), migrate.Validate(d))
+	require.Equal(t, edited(3, 2, 115, "1_initial.up.sql"), migrate.Validate(d))
 	require.NoError(t, os.WriteFile(filepath.Join(td, "1_initial.up.sql"), initialUp, 0644))
 
 	// Adding a file at the end.
 	require.NoError(t, os.WriteFile(filepath.Join(td, "2_second.sql"), []byte("stmt"), os.ModePerm))
 	t.Cleanup(func() { os.Remove(filepath.Join(td, "2_second.sql")) })
-	require.Equal(t, added(3, 2, 180, "2_second.sql"), migrate.Validate(d))
+	require.Equal(t, added(4, 2, 180, "2_second.sql"), migrate.Validate(d))
 	require.NoError(t, os.Remove(filepath.Join(td, "2_second.sql")))
 
 	// Changing the filename should raise validation error.
@@ -142,14 +144,14 @@ func TestValidate(t *testing.T) {
 	t.Cleanup(func() {
 		require.NoError(t, os.Rename(filepath.Join(td, "1_first.up.sql"), filepath.Join(td, "1_initial.up.sql")))
 	})
-	require.Equal(t, added(1, 2, 48, "1_first.up.sql"), migrate.Validate(d))
+	require.Equal(t, added(2, 2, 48, "1_first.up.sql"), migrate.Validate(d))
 
 	// Removing it as well (move it out of the dir).
 	require.NoError(t, os.Rename(filepath.Join(td, "1_first.up.sql"), filepath.Join(td, "..", "bak")))
 	t.Cleanup(func() {
 		require.NoError(t, os.Rename(filepath.Join(td, "..", "bak"), filepath.Join(td, "1_first.up.sql")))
 	})
-	require.Equal(t, removed(2, 2, 115, "1_initial.up.sql"), migrate.Validate(d))
+	require.Equal(t, removed(3, 2, 115, "1_initial.up.sql"), migrate.Validate(d))
 }
 
 func TestHash_MarshalText(t *testing.T) {
@@ -512,6 +514,65 @@ func TestDirTar(t *testing.T) {
 	require.Len(t, files, 1)
 	require.Equal(t, "1.sql", files[0].Name())
 	require.Equal(t, "create table t(c int);", string(files[0].Bytes()))
+
+	// Compress the dir.
+	var buf bytes.Buffer
+	w := gzip.NewWriter(&buf)
+	require.NoError(t, migrate.ArchiveDirTo(w, d))
+	require.NoError(t, w.Close()) // flush and Close.
+
+	// Decompress.
+	rr, err := gzip.NewReader(&buf)
+	require.NoError(t, err)
+	dir, err = migrate.UnarchiveDirFrom(rr)
+	require.NoError(t, err)
+	files, err = dir.Files()
+	require.Len(t, files, 1)
+	require.Equal(t, "1.sql", files[0].Name())
+	require.Equal(t, "create table t(c int);", string(files[0].Bytes()))
+}
+
+func TestDefaultFormatter_FormatTo(t *testing.T) {
+	var b bytes.Buffer
+	err := migrate.DefaultFormatter.FormatTo(&migrate.Plan{
+		Changes: []*migrate.Change{
+			{Cmd: "create table t1(c int)"},
+			{Cmd: "create table t2(c int)", Comment: "create table"},
+		},
+	}, &b)
+	require.NoError(t, err)
+	require.Equal(t, `create table t1(c int);
+-- Create table
+create table t2(c int);
+`, b.String())
+}
+
+func TestDefaultFormatter_FormatFile(t *testing.T) {
+	f, err := migrate.DefaultFormatter.FormatFile(&migrate.Plan{
+		Changes: []*migrate.Change{
+			{Cmd: "create table t1(c int)"},
+			{Cmd: "create table t2(c int)", Comment: "create table"},
+		},
+	})
+	require.NoError(t, err)
+	require.Equal(t, `create table t1(c int);
+-- Create table
+create table t2(c int);
+`, string(f.Bytes()))
+
+	ft := migrate.TemplateFormatter{
+		{
+			N: template.Must(template.New("").Parse("name-1")),
+			C: template.Must(template.New("").Parse("cmd-1")),
+		},
+		{
+			N: template.Must(template.New("").Parse("name-2")),
+			C: template.Must(template.New("").Parse("cmd-2")),
+		},
+	}
+	f, err = ft.FormatFile(&migrate.Plan{})
+	require.Error(t, err, "expect only one file")
+	require.Nil(t, f)
 }
 
 func fileNames(r io.Reader) ([]string, error) {
