@@ -63,6 +63,11 @@ func (d *diff) TableAttrDiff(from, to *schema.Table) ([]schema.Change, error) {
 	if err := d.partitionChanged(from, to); err != nil {
 		return nil, err
 	}
+	change, err := d.tableAttrDiff(from, to)
+	if err != nil {
+		return nil, err
+	}
+	changes = append(changes, change...)
 	return append(changes, sqlx.CheckDiff(from, to, func(c1, c2 *schema.Check) bool {
 		return sqlx.Has(c1.Attrs, &NoInherit{}) == sqlx.Has(c2.Attrs, &NoInherit{})
 	})...), nil
@@ -119,7 +124,7 @@ func (d *diff) defaultChanged(from, to *schema.Column) (bool, error) {
 	//	SELECT ARRAY[1] = '{1}'::int[]
 	//	SELECT lower('X'::text) = lower('X')
 	//
-	if (fromX || toX) && d.conn.ExecQuerier != nil {
+	if (fromX || toX) && d.conn.ExecQuerier != nil && d.conn.ExecQuerier != sqlx.NoRows {
 		equals, err := d.defaultEqual(from.Default, to.Default)
 		return !equals, err
 	}
@@ -324,7 +329,7 @@ func typeChanged(from, to *schema.Column, ns string) (bool, error) {
 	switch fromT := fromT.(type) {
 	case *schema.BinaryType, *BitType, *schema.BoolType, *schema.DecimalType, *schema.FloatType, *IntervalType,
 		*schema.IntegerType, *schema.JSONType, *OIDType, *RangeType, *SerialType, *schema.SpatialType,
-		*schema.StringType, *schema.TimeType, *TextSearchType, *NetworkType, *schema.UUIDType:
+		*schema.StringType, *PseudoType, *schema.TimeType, *TextSearchType, *NetworkType, *schema.UUIDType:
 		t1, err := FormatType(toT)
 		if err != nil {
 			return false, err
@@ -340,6 +345,10 @@ func typeChanged(from, to *schema.Column, ns string) (bool, error) {
 			// In case the type is defined with schema qualifier, but returned without
 			// (inspecting a schema scope), or vice versa, remove before comparing.
 			ns != "" && trimSchema(toT.T, ns) != trimSchema(toT.T, ns)
+	case *CompositeType:
+		toT := toT.(*CompositeType)
+		changed = toT.T != fromT.T ||
+			(toT.Schema != nil && fromT.Schema != nil && toT.Schema.Name != fromT.Schema.Name)
 	case *DomainType:
 		toT := toT.(*DomainType)
 		changed = toT.T != fromT.T ||
