@@ -811,6 +811,19 @@ func TestPlanChanges(t *testing.T) {
 									},
 								},
 							},
+							&schema.AddIndex{
+								I: &schema.Index{
+									Unique: true,
+									Name:   "unique_const_nulls_not_distinct",
+									Parts: []*schema.IndexPart{
+										{C: users.Columns[0]},
+									},
+									Attrs: []schema.Attr{
+										&IndexNullsDistinct{V: false},
+										&Constraint{N: "unique_const", T: "u"},
+									},
+								},
+							},
 						},
 					}
 				}(),
@@ -824,8 +837,8 @@ func TestPlanChanges(t *testing.T) {
 						Reverse: `CREATE INDEX CONCURRENTLY "drop_con" ON "users" ("id")`,
 					},
 					{
-						Cmd:     `ALTER TABLE "users" DROP CONSTRAINT "id_nonzero", ADD COLUMN "name" character varying(255) NOT NULL DEFAULT 'logged_in', ADD COLUMN "last" character varying(255) NOT NULL DEFAULT 'logged_in', ADD CONSTRAINT "name_not_empty" CHECK ("name" <> ''), ADD CONSTRAINT "positive_id" CHECK ("id" > 0) NOT VALID, DROP CONSTRAINT "id_iseven", ADD CONSTRAINT "id_iseven" CHECK (("id") % 2 = 0)`,
-						Reverse: `ALTER TABLE "users" DROP CONSTRAINT "id_iseven", ADD CONSTRAINT "id_iseven" CHECK ("id" % 2 = 0), DROP CONSTRAINT "positive_id", DROP CONSTRAINT "name_not_empty", DROP COLUMN "last", DROP COLUMN "name", ADD CONSTRAINT "id_nonzero" CHECK ("id" <> 0)`,
+						Cmd:     `ALTER TABLE "users" DROP CONSTRAINT "id_nonzero", ADD COLUMN "name" character varying(255) NOT NULL DEFAULT 'logged_in', ADD COLUMN "last" character varying(255) NOT NULL DEFAULT 'logged_in', ADD CONSTRAINT "name_not_empty" CHECK ("name" <> ''), ADD CONSTRAINT "positive_id" CHECK ("id" > 0) NOT VALID, DROP CONSTRAINT "id_iseven", ADD CONSTRAINT "id_iseven" CHECK (("id") % 2 = 0), ADD CONSTRAINT "unique_const" UNIQUE NULLS NOT DISTINCT ("id")`,
+						Reverse: `ALTER TABLE "users" DROP CONSTRAINT "unique_const_nulls_not_distinct", DROP CONSTRAINT "id_iseven", ADD CONSTRAINT "id_iseven" CHECK ("id" % 2 = 0), DROP CONSTRAINT "positive_id", DROP CONSTRAINT "name_not_empty", DROP COLUMN "last", DROP COLUMN "name", ADD CONSTRAINT "id_nonzero" CHECK ("id" <> 0)`,
 					},
 					{
 						Cmd:     `CREATE INDEX "id_key" ON "users" ("id" DESC) WHERE success`,
@@ -856,12 +869,12 @@ func TestPlanChanges(t *testing.T) {
 						Reverse: `DROP INDEX "nulls_not_distinct"`,
 					},
 					{
-						Cmd:     `COMMENT ON COLUMN "users" ."name" IS 'foo'`,
-						Reverse: `COMMENT ON COLUMN "users" ."name" IS ''`,
+						Cmd:     `COMMENT ON COLUMN "users"."name" IS 'foo'`,
+						Reverse: `COMMENT ON COLUMN "users"."name" IS ''`,
 					},
 					{
-						Cmd:     `COMMENT ON COLUMN "users" ."last" IS 'bar'`,
-						Reverse: `COMMENT ON COLUMN "users" ."last" IS ''`,
+						Cmd:     `COMMENT ON COLUMN "users"."last" IS 'bar'`,
+						Reverse: `COMMENT ON COLUMN "users"."last" IS ''`,
 					},
 					{
 						Cmd:     `COMMENT ON INDEX "id_key" IS 'comment'`,
@@ -903,8 +916,8 @@ func TestPlanChanges(t *testing.T) {
 						Reverse: `ALTER TABLE "users" ALTER COLUMN "id" SET GENERATED BY DEFAULT SET START WITH 1 SET INCREMENT BY 1 RESTART, ADD COLUMN "name" character varying NOT NULL`,
 					},
 					{
-						Cmd:     `COMMENT ON COLUMN "users" ."id" IS ''`,
-						Reverse: `COMMENT ON COLUMN "users" ."id" IS 'comment'`,
+						Cmd:     `COMMENT ON COLUMN "users"."id" IS ''`,
+						Reverse: `COMMENT ON COLUMN "users"."id" IS 'comment'`,
 					},
 				},
 			},
@@ -1293,7 +1306,7 @@ func TestPlanChanges(t *testing.T) {
 				Transactional: true,
 				Changes: []*migrate.Change{
 					{Cmd: `ALTER TYPE "public"."state" ADD VALUE 'unknown'`},
-					{Cmd: `COMMENT ON COLUMN "public"."users" ."state" IS ''`, Reverse: `COMMENT ON COLUMN "public"."users" ."state" IS 'foo'`},
+					{Cmd: `COMMENT ON COLUMN "public"."users"."state" IS ''`, Reverse: `COMMENT ON COLUMN "public"."users"."state" IS 'foo'`},
 				},
 			},
 		},
@@ -1323,7 +1336,7 @@ func TestPlanChanges(t *testing.T) {
 				Reversible:    true,
 				Transactional: true,
 				Changes: []*migrate.Change{
-					{Cmd: `COMMENT ON COLUMN "users" ."state" IS 'foo'`, Reverse: `COMMENT ON COLUMN "users" ."state" IS 'bar'`},
+					{Cmd: `COMMENT ON COLUMN "users"."state" IS 'foo'`, Reverse: `COMMENT ON COLUMN "users"."state" IS 'bar'`},
 				},
 			},
 		},
@@ -1489,6 +1502,9 @@ func TestPlanChanges(t *testing.T) {
 					},
 				},
 			},
+			options: []migrate.PlanOption{
+				func(o *migrate.PlanOptions) { o.SchemaQualifier = new(string) },
+			},
 			wantPlan: &migrate.Plan{
 				Reversible:    true,
 				Transactional: true,
@@ -1500,6 +1516,29 @@ func TestPlanChanges(t *testing.T) {
 				},
 			},
 		},
+		{
+			changes: []schema.Change{
+				&schema.ModifyTable{
+					T: schema.NewTable("t1").SetSchema(schema.New("s1")),
+					Changes: []schema.Change{
+						&schema.RenameIndex{
+							From: schema.NewIndex("a"),
+							To:   schema.NewIndex("b"),
+						},
+					},
+				},
+			},
+			wantPlan: &migrate.Plan{
+				Reversible:    true,
+				Transactional: true,
+				Changes: []*migrate.Change{
+					{
+						Cmd:     `ALTER INDEX "s1"."a" RENAME TO "b"`,
+						Reverse: `ALTER INDEX "s1"."b" RENAME TO "a"`,
+					},
+				},
+			},
+		},
 		// Invalid serial type.
 		{
 			changes: []schema.Change{
