@@ -65,6 +65,13 @@ table "table" {
 		where = "active"
 		comment = "index comment"
 	}
+	index "index_hnsw" {
+		type = "HNSW"
+		columns = [
+			table.table.column.col,
+			table.table.column.age,
+		]
+	}
 	foreign_key "accounts" {
 		columns = [
 			table.table.column.account_name,
@@ -240,6 +247,17 @@ enum "account_type" {
 				&IndexPredicate{P: "active"},
 			},
 		},
+		{
+			Name:  "index_hnsw",
+			Table: exp.Tables[0],
+			Parts: []*schema.IndexPart{
+				{SeqNo: 0, C: exp.Tables[0].Columns[0]},
+				{SeqNo: 1, C: exp.Tables[0].Columns[1]},
+			},
+			Attrs: []schema.Attr{
+				&IndexType{T: "HNSW"},
+			},
+		},
 	}
 	exp.Tables[0].ForeignKeys = []*schema.ForeignKey{
 		{
@@ -540,9 +558,51 @@ table "t" {
 		require.NoError(t, EvalHCLBytes([]byte(r), &s, nil))
 		idx = s.Tables[0].Indexes[0]
 		require.Equal(t, IndexTypeGiST, idx.Attrs[0].(*IndexType).T)
+
+		s = schema.Schema{}
+		r = fmt.Sprintf(f, `"HNSW"`)
+		require.NoError(t, EvalHCLBytes([]byte(r), &s, nil))
+		idx = s.Tables[0].Indexes[0]
+		require.Equal(t, "HNSW", idx.Attrs[0].(*IndexType).T)
 	})
 }
 
+func TestMarshalSpec_IndexType(t *testing.T) {
+	s := schema.New("public").
+		AddTables(
+			schema.NewTable("t1").
+				AddColumns(
+					schema.NewIntColumn("id", "int"),
+				).
+				AddIndexes(
+					schema.NewIndex("i1").AddAttrs(&IndexType{T: IndexTypeHash}),
+					schema.NewIndex("i2").AddAttrs(&IndexType{T: "HNSW"}),
+					schema.NewIndex("i2").AddAttrs(&IndexType{T: IndexTypeBTree}), // Default.
+				),
+		)
+	buf, err := MarshalHCL(s)
+	require.NoError(t, err)
+	require.Equal(t, `table "t1" {
+  schema = schema.public
+  column "id" {
+    null = false
+    type = int
+  }
+  index "i1" {
+    type = HASH
+  }
+  index "i2" {
+    type = "HNSW"
+  }
+  index "i2" {
+  }
+}
+schema "public" {
+}
+`,
+		string(buf))
+}
+
 func TestUnmarshalSpec_BRINIndex(t *testing.T) {
 	f := `
 schema "s" {}
@@ -610,6 +670,13 @@ func TestUnmarshalSpec_IndexOpClass(t *testing.T) {
       ops    = sql("tsvector_ops(siglen=1)")
     }
   }
+  index "idx5" {
+    unique = true
+    on {
+      column = column.a
+      ops    = "vector_cosine_ops"
+    }
+  }
 }
 schema "test" {
 }
@@ -653,6 +720,12 @@ schema "test" {
 	require.Len(t, opc.Params, 1)
 	require.Equal(t, "siglen", opc.Params[0].N)
 	require.Equal(t, "1", opc.Params[0].V)
+	// idx5
+	idx = s.Tables[0].Indexes[5]
+	require.Len(t, idx.Parts, 1)
+	require.Equal(t, "a", idx.Parts[0].C.Name)
+	require.Len(t, idx.Parts[0].Attrs, 1)
+	require.Equal(t, "vector_cosine_ops", idx.Parts[0].Attrs[0].(*IndexOpClass).Name)
 }
 
 func TestUnmarshalSpec_Partitioned(t *testing.T) {
@@ -905,6 +978,71 @@ schema "public" {
 	require.EqualValues(t, expected, string(buf))
 }
 
+func TestMarshalSpec_IndexNullsLastFirst(t *testing.T) {
+	s := schema.New("public").
+		AddTables(
+			schema.NewTable("users").
+				AddColumns(
+					schema.NewIntColumn("c", "int"),
+				).
+				AddIndexes(
+					// Default behavior.
+					schema.NewIndex("i1").
+						AddParts(schema.NewColumnPart(schema.NewColumn("c")).SetDesc(true).AddAttrs(&IndexColumnProperty{NullsFirst: true})),
+					schema.NewIndex("i2").
+						AddParts(schema.NewColumnPart(schema.NewColumn("c")).SetDesc(false).AddAttrs(&IndexColumnProperty{NullsLast: true})),
+					// Require emitting the NULLS FIRST/LAST attributes.
+					schema.NewIndex("i3").
+						AddParts(schema.NewColumnPart(schema.NewColumn("c")).SetDesc(true).AddAttrs(&IndexColumnProperty{NullsLast: true})),
+					schema.NewIndex("i4").
+						AddParts(schema.NewColumnPart(schema.NewColumn("c")).SetDesc(false).AddAttrs(&IndexColumnProperty{NullsFirst: true})),
+				),
+		)
+	buf, err := MarshalSpec(s, hclState)
+	require.NoError(t, err)
+	const expected = `table "users" {
+  schema = schema.public
+  column "c" {
+    null = false
+    type = int
+  }
+  index "i1" {
+    on {
+      desc   = true
+      column = column.c
+    }
+  }
+  index "i2" {
+    columns = [column.c]
+  }
+  index "i3" {
+    on {
+      desc       = true
+      column     = column.c
+      nulls_last = true
+    }
+  }
+  index "i4" {
+    on {
+      column      = column.c
+      nulls_first = true
+    }
+  }
+}
+schema "public" {
+}
+`
+	require.EqualValues(t, expected, string(buf))
+
+	var got schema.Schema
+	err = EvalHCLBytes([]byte(expected), &got, nil)
+	require.NoError(t, err)
+
+	buf, err = MarshalSpec(s, hclState)
+	require.NoError(t, err)
+	require.EqualValues(t, expected, string(buf))
+}
+
 func TestMarshalSpec_BRINIndex(t *testing.T) {
 	s := &schema.Schema{
 		Name: "test",
@@ -1036,6 +1174,14 @@ func TestMarshalSpec_IndexOpClass(t *testing.T) {
 				&IndexType{T: IndexTypeGiST},
 			},
 		},
+		{
+			Name:   "idx5",
+			Table:  s.Tables[0],
+			Unique: true,
+			Parts: []*schema.IndexPart{
+				{SeqNo: 0, C: s.Tables[0].Columns[0], Attrs: []schema.Attr{&IndexOpClass{Name: "vector_cosine_ops"}}},
+			},
+		},
 	}
 	buf, err := MarshalSpec(s, hclState)
 	require.NoError(t, err)
@@ -1082,6 +1228,13 @@ func TestMarshalSpec_IndexOpClass(t *testing.T) {
       ops    = sql("tsvector_ops(siglen=1)")
     }
   }
+  index "idx5" {
+    unique = true
+    on {
+      column = column.a
+      ops    = "vector_cosine_ops"
+    }
+  }
 }
 schema "test" {
 }
@@ -1521,6 +1674,10 @@ func TestTypes(t *testing.T) {
 			typeExpr: "text",
 			expected: &schema.StringType{T: TypeText},
 		},
+		{
+			typeExpr: "bpchar",
+			expected: &schema.StringType{T: TypeBPChar},
+		},
 		{
 			typeExpr: "smallint",
 			expected: &schema.IntegerType{T: TypeSmallInt},
@@ -1829,6 +1986,14 @@ func TestTypes(t *testing.T) {
 			typeExpr: "name",
 			expected: &schema.StringType{T: typeName},
 		},
+		{
+			typeExpr: "xid",
+			expected: &schema.IntegerType{T: TypeXID},
+		},
+		{
+			typeExpr: "xid8",
+			expected: &schema.IntegerType{T: TypeXID8},
+		},
 	} {
 		t.Run(tt.typeExpr, func(t *testing.T) {
 			var test schema.Schema
@@ -2238,3 +2403,92 @@ schema "s3" {
 }
 `, string(buf))
 }
+
+func TestMarshalSpec_UniqueConstraint(t *testing.T) {
+	s := schema.New("public").
+		AddTables(
+			schema.NewTable("t1").
+				AddColumns(
+					schema.NewIntColumn("id", "int"),
+					schema.NewIntColumn("uid", "int"),
+				).
+				AddIndexes(
+					schema.NewUniqueIndex("u1").AddColumns(schema.NewColumn("id")).AddAttrs(UniqueConstraint("u1")),
+					schema.NewUniqueIndex("u2").AddColumns(schema.NewColumn("id")).AddAttrs(UniqueConstraint("u2"), &IndexNullsDistinct{}),
+					schema.NewUniqueIndex("u3").AddColumns(schema.NewColumn("id")).AddAttrs(UniqueConstraint("u3"), &IndexInclude{Columns: []*schema.Column{schema.NewColumn("uid")}}),
+				),
+		)
+	buf, err := MarshalHCL(s)
+	require.NoError(t, err)
+	require.Equal(t, `table "t1" {
+  schema = schema.public
+  column "id" {
+    null = false
+    type = int
+  }
+  column "uid" {
+    null = false
+    type = int
+  }
+  unique "u1" {
+    columns = [column.id]
+  }
+  unique "u2" {
+    columns        = [column.id]
+    nulls_distinct = false
+  }
+  unique "u3" {
+    columns = [column.id]
+    include = [column.uid]
+  }
+}
+schema "public" {
+}
+`,
+		string(buf))
+}
+
+func TestUnmarshalSpec_UniqueConstraint(t *testing.T) {
+	var s schema.Schema
+	require.NoError(t, EvalHCLBytes([]byte(`table "t1" {
+  schema = schema.public
+  column "id" {
+    null = false
+    type = int
+  }
+  column "uid" {
+    null = false
+    type = int
+  }
+  unique "u1" {
+    columns = [column.id]
+  }
+  unique "u2" {
+    columns        = [column.id]
+    nulls_distinct = false
+  }
+  unique "u3" {
+    columns = [column.id]
+    include = [column.uid]
+  }
+}
+schema "public" {
+}
+`), &s, nil))
+	require.Len(t, s.Tables, 1)
+	require.Len(t, s.Tables[0].Indexes, 3)
+	// u1
+	u1 := s.Tables[0].Indexes[0]
+	require.Equal(t, "u1", u1.Name)
+	require.Equal(t, UniqueConstraint("u1"), u1.Attrs[0].(*Constraint))
+	// u2
+	u2 := s.Tables[0].Indexes[1]
+	require.Equal(t, "u2", u2.Name)
+	require.Equal(t, &IndexNullsDistinct{}, u2.Attrs[0])
+	require.Equal(t, UniqueConstraint("u2"), u2.Attrs[1].(*Constraint))
+	// u3
+	u3 := s.Tables[0].Indexes[2]
+	require.Equal(t, "u3", u3.Name)
+	require.Equal(t, &IndexInclude{Columns: []*schema.Column{s.Tables[0].Columns[1]}}, u3.Attrs[0])
+	require.Equal(t, UniqueConstraint("u3"), u3.Attrs[1].(*Constraint))
+}
