@@ -5,6 +5,9 @@
 package sqlspec
 
 import (
+	"bytes"
+	"strings"
+
 	"ariga.io/atlas/schemahcl"
 
 	"github.com/zclconf/go-cty/cty"
@@ -122,6 +125,16 @@ type (
 		// Attributes and blocks are different for each driver.
 		schemahcl.DefaultExtension
 	}
+
+	// Sequence holds a specification for a Sequence.
+	Sequence struct {
+		Name      string         `spec:",name"`
+		Qualifier string         `spec:",qualifier"`
+		Schema    *schemahcl.Ref `spec:"schema"`
+		// Type, Start, Increment, Min, Max, Cache, Cycle
+		// are optionally added to the sequence definition.
+		schemahcl.DefaultExtension
+	}
 )
 
 // Label returns the defaults label used for the table resource.
@@ -160,6 +173,18 @@ func (f *Func) SetQualifier(q string) { f.Qualifier = q }
 // SchemaRef returns the schema reference for the function.
 func (f *Func) SchemaRef() *schemahcl.Ref { return f.Schema }
 
+// Label returns the defaults label used for the sequence resource.
+func (s *Sequence) Label() string { return s.Name }
+
+// QualifierLabel returns the qualifier label used for the sequence resource, if any.
+func (s *Sequence) QualifierLabel() string { return s.Qualifier }
+
+// SetQualifier sets the qualifier label used for the sequence resource.
+func (s *Sequence) SetQualifier(q string) { s.Qualifier = q }
+
+// SchemaRef returns the schema reference for the sequence.
+func (s *Sequence) SchemaRef() *schemahcl.Ref { return s.Schema }
+
 func init() {
 	schemahcl.Register("view", &View{})
 	schemahcl.Register("materialized", &View{})
@@ -167,5 +192,39 @@ func init() {
 	schemahcl.Register("function", &Func{})
 	schemahcl.Register("procedure", &Func{})
 	schemahcl.Register("trigger", &Trigger{})
+	schemahcl.Register("sequence", &Sequence{})
 	schemahcl.Register("schema", &Schema{})
 }
+
+// normalizeCRLF for heredoc strings that inspected and printed in the HCL as-is to
+// avoid having mixed endings in the printed file - Unix-like (default) and Windows-like.
+func normalizeCRLF(s string) string {
+	if strings.Contains(s, "\r\n") {
+		return strings.ReplaceAll(s, "\r\n", "\n")
+	}
+	return s
+}
+
+// MightHeredoc returns the string as an indented heredoc if it has multiple lines.
+func MightHeredoc(s string) string {
+	s = normalizeCRLF(strings.TrimSpace(s))
+	// In case the given definition is multi-line,
+	// format it as indented heredoc with two spaces.
+	if lines := strings.Split(s, "\n"); len(lines) > 1 {
+		var b bytes.Buffer
+		b.Grow(len(s))
+		b.WriteString("<<-SQL\n")
+		for _, l := range lines {
+			// Skip spaces-only lines, as editors stripped these spaces off,
+			// and HCL parser results a different string for them.
+			if strings.TrimSpace(l) != "" {
+				b.WriteString("  ")
+				b.WriteString(l)
+			}
+			b.WriteByte('\n')
+		}
+		b.WriteString("  SQL")
+		s = b.String()
+	}
+	return s
+}
